﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.DragDrop;
using System.Windows.Data;
using System.Collections;
using Telerik.Windows.Controls;
using System.Windows.Media;

namespace DragDropWithinListBox_WPF
{
    public class DragHelper
    {

        public RadListBoxItem DraggedListBoxItem { get; set; }
        public RadListBox ListBoxElement { get; set; }
        
        public static bool GetReorderOnDrop(DependencyObject obj)
        {
            return (bool)obj.GetValue(ReorderOnDropProperty);
        }

        public static void SetReorderOnDrop(DependencyObject obj, bool value)
        {
           
            DragHelper behavior = new DragHelper();
            behavior.DraggedListBoxItem = obj as RadListBoxItem;

            if (value)
            {
                behavior.Initialize() ;
            }
            else
            {
                behavior.CleanUp() ;
            }

            obj.SetValue(ReorderOnDropProperty, value);
        }

        // Using a DependencyProperty as the backing store for ReorderOnDrop.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ReorderOnDropProperty =
            DependencyProperty.RegisterAttached("ReorderOnDrop", typeof(bool), typeof(DragHelper), new PropertyMetadata(false, OnReorderOnDropPropertyChanged));

        private static void OnReorderOnDropPropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SetReorderOnDrop(dependencyObject, (bool)e.NewValue);
        }

        protected virtual void Initialize()
        {
            this.UnsubscribeFromDragDropEvents();
            this.SubscribeToDragDropEvents();
        }

        protected virtual void CleanUp()
        {
            this.UnsubscribeFromDragDropEvents();
        }

        private void SubscribeToDragDropEvents()
        {
            DragDropManager.AddDragInitializeHandler(this.DraggedListBoxItem, OnDragInitialize);
            DragDropManager.AddDropHandler(this.DraggedListBoxItem, OnDrop);
        }


        private void UnsubscribeFromDragDropEvents()
        {
            DragDropManager.RemoveDragInitializeHandler(this.DraggedListBoxItem, OnDragInitialize);
            DragDropManager.RemoveDropHandler(this.DraggedListBoxItem, OnDrop);
        }

        private void OnDragInitialize(object sender, DragInitializeEventArgs args)
        {
            args.AllowedEffects = DragDropEffects.All;
            var payload = DragDropPayloadManager.GeneratePayload(null);
            var data = ((FrameworkElement)args.OriginalSource).DataContext;
            payload.SetData(typeof(ListBoxItemViewModel), data);
            args.Data = payload;

            args.DragVisual = new ContentControl
            {
                ContentTemplate = Application.Current.Resources["DragVisualTemplate"] as DataTemplate,
                Content = data,
            };
        }

        private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
        {
            int counter = 0;
            var data = args.Data as IDataObject;

            //the dragged item:
            var listBoxItem = data.GetData(typeof(ListBoxItemViewModel)) as ListBoxItemViewModel;
            
            //the item on which is dropped:
            var itemContainer = sender as RadListBoxItem;
            var itemsControl = ItemsControl.ItemsControlFromItemContainer(itemContainer);

            foreach (ListBoxItemViewModel item in itemsControl.ItemsSource)
            {
                if (itemContainer.DataContext == item)
                {
                    ((IList)itemsControl.ItemsSource).Remove(listBoxItem);
                    ((IList)itemsControl.ItemsSource).Insert(counter, listBoxItem);
                    break;
                }

                counter++;
            }

            args.Handled = true;
        }
    }
}
